<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2005 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Frederic Wenzel <fred@osuosl.org>                           |
// |            Zack Bartel <bartel@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id$

/**
  * Backend.php - contains the {@link Backend} class
  *
  * @author $Author$
  * @filesource
  *
  * @package    classes
  * @subpackage backend
  */

require_once('PEAR.php');

/**
 * Backend class provides main backend functionality. It is usually
 * instantiated and triggered by the maintain-update cron jobs
 *
 * @package classes
 * @subpackage backend
 *
 * @final
 */
class Backend
{
// {{{ vars
    var $force = false; // force the build process to run?
// }}}
// {{{ constructor
    function Backend()
    {
        // set error handling to our callback function
        PEAR::setErrorHandling(PEAR_ERROR_CALLBACK, array(&$this, 'handleErr'));
    }
// }}}
// {{{ run()
/**
 * runs all backend processes sequentially
 *
 * @return void
 */
    function run()
    {
        if (!$this->force && !$this->verifyBuildTimestamp(true)) {
            PEAR::raiseError(_('Backend build process could not be started '
                .'because there is another one currently running. Please '
                .'try again later or if you are an admin, you may force the '
                .'process with the -f flag on the command line.'));
            return;
        }

        Module::do_action('backend_backup');
        Module::do_action('backend_maintenance');
        Module::do_action('backend_build');
        Module::do_action('backend_push');

        $this->_wipeLock();
    }
// }}}

//{{{ verifyBuildTimestamp()
    /**
     * Verifies that no other backend process is currently running.
     *
     * @param   Bool If there is no other lock, do you want me to
     *          acquire a new lock for you?
     * @return  Bool true if no backend process is running or backend
     *          timed out before.
     * @global  Maintain config array
     */
    function verifyBuildTimestamp($acquireLock = true)
    {
        global $conf;
        /*
        $dbdo = DB_Dataobject::Factory('update_info'); 
        $dbdo->query("SELECT * FROM update_info WHERE `key` = 'backend_build_start'");
        $dbdo->fetch();
        */
        
        // TODO: Note: DB_dataobject doesn't like field names that use 
        // reserved words, e.g. 'key'
        // So we have to use plain text queries here until this table is
        // fixed.
        
        $res = mysql_query("SELECT * FROM update_info WHERE `key` = 'backend_build_start'", dbh());
        if (mysql_num_rows($res) == 0) {
            if ($acquireLock)
                $this->_setupRunTime(true);
            return true;
        }
        
        // default timeout: 15 min
        if (!isset($conf['general']['build_timeout']) 
            || !($conf['general']['build_timeout'] >= 0)) {
            $timeout = 900;
        } else {
            $timeout = $conf['general']['build_timeout'];
        }
        
        $row = mysql_fetch_array($res);
        if ((int)$row['value'] <= time() - $timeout) {
            if ($acquireLock)
                $this->_setupRunTime(false);
            return true;
        }

        return false;
    }
//}}}
//{{{ _setupRunTime()
    /**
     * Adds initial entry to the update_info table with a time value of 0
     *
     * @param   Bool insert new field, defaults to true, otherwise update
     *          existing value
     * @return  Bool true on success else false
     * @private
     */
    function _setupRunTime($use_insert = true)
    {
        if ($use_insert) {
            $sql = "INSERT INTO `update_info` (`key`, `value`) VALUES ('backend_build_start', '".time()."');";
        } else {
            $sql = "UPDATE `update_info` SET `value` = '".time()."' WHERE `key` = 'backend_build_start';";
        }
        $dbdo = DB_Dataobject::Factory('update_info'); 
        $dbdo->query($sql);
    }
//}}}
//{{{ _wipeLock()
    /**
     * Remove the lock from the database (after completion of the build
     * process)
     *
     * @return  true
     *
     */
    function _wipeLock()
    {
        $dbdo = DB_Dataobject::Factory('update_info'); 
        $dbdo->query("UPDATE `update_info` SET `value` = '0' WHERE `key` = 'backend_build_start';");

        return true;
    }
//}}}
// {{{ handleErr()
/**
 * handles PEAR errors risen during Backend operations.
 * currently dumps everything to stderr. Might be changed, though.
 *
 * @param PEAR_Error the error that was risen
 * @return void
 */
    function handleErr($error)
    {
        $stderr = fopen('php://stderr', 'w');
        fwrite($stderr, "ERROR: ".$error->getMessage()."\n");
        fclose($stderr);
    }
/// }}}

}
