<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Authors: Michael Clay <claym@osuosl.org>                               |
// |          Frederic Wenzel <fred@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id $

require_once("$phphtmllib/widgets/data_list/DefaultGUIDataList.inc");
require_once(DAOEXT.'/domain.php');
require_once(HTCLASS.'/DB_DataObjectDataListSource.php');

/**
 *  This class is the extension of DefaultGUIDataList made for use with the pointer
 *  table.
 */
class DomainDataList extends DefaultGUIDataList
{

//{{{ vars

    var $_default_rows_per_page = 38;

//}}}
//{{{   get_data_source()

    /**
     *  This method sets up the data source for the data list to be displayed.
     *  In Maintain's case, this means creating DBDataObject Data Source objects.
     */
    function get_data_source()
    {
        global $conf;

        $domain = new Domain();
        $zd = DB_DataObject::Factory('zone_domain');
        $zd->joinAdd($domain);
        $src = &new DB_DataObjectDataListSource($zd);
        $this->set_data_source($src);
        $this->_default_orderby = 'name';
    }

//}}}
//{{{   user_setup()

    /**
     *  This method sets up the table columns and various other attributes like
     *  which of those columns is sortable, searchable, etc...
     */
    function user_setup()
    {
        global $conf;

        $this->set_actionbar(true);
        $access = true;

        $this->add_header_item(_("Domain Name"), '20%', 'name', SORTABLE, SEARCHABLE);
       

        $this->add_header_item(_("Parent Zone"), '20%');

        $this->add_header_item(_("Status"),'20%','enabled',SORTABLE,null);

        $this->add_header_item(_("Action"),'20%');
        $this->_datasource->setup_db_options(array('zone' => $_SESSION['zone']));
        $this->search_disable();        
    }

//}}}
//{{{   build_column_item()

    /**
     *  This method takes care of how the data in the columns will appear.
     *
     *  @param  array   $row_data   the row in question
     *  @param  string  $col_name   the column in question
     *
     *  @return Object  $obj        the custom rendering of the data
     */
    function build_column_item($row_data, $col_name)
    {
        global $conf;

        switch($col_name) {
            case _('Status'):
                $obj = ($row_data['enabled']?_('Enabled'):_('Disabled'));
                break;
            case _('Action'): //static row is just links
                $edit = html_a(web_root()."domain.php?action=edit&amp;id=".$row_data['id'], "Edit");
                $delete = html_a(web_root()."domain.php?action=delete&amp;id=".$row_data['id'],"Delete");
                $enable = html_a(web_root()."domain.php?action=toggle_enable&amp;id=".$row_data['id'],($row_data['enabled']?_('Disable'):_('Enable')));
                $obj = container($edit,"|",$enable,"|",$delete);
                break;
            case _('Parent Zone'):
                $zd = DB_DataObject::Factory('zone_domain');
                $zd->domain = $row_data['domain'];
                //find all occurences of this domain in zone_domain
                $res = $zd->find();
                $ary = array();
                while($zd->fetch()) {
                    $zn = DB_DataObject::Factory('zone');
                    $zn->id = $zd->zone;
                    $zn->find(1);
                    $ary[$zn->name] = $zn->id;
                }
                $obj = form_select("parent_zone", $ary, $_SESSION['zone']);
                break;
            case _('Domain Name'):
                $obj = container(html_a(web_root()."domain.php?action=show&amp;id=".$row_data['id'], $row_data['name']));

                break;
                
            default:
                $obj = DefaultGUIDataList::build_column_item($row_data, $col_name);
        }

        return $obj;
    }

//}}}
//{{{   actionbar_cell()

    /**
     *  This method is what sets the information display at the bottom of the table.
     *  
     *  @return Container   A span that has strings for displaying the page numbers
     */
    function actionbar_cell()
    {
        $current = $this->get_current_page();
        $pages = $this->get_num_pages();
        return html_span('',_("You are on page ").$current." of ".$pages);
    }

//}}}

}

?>
