<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Zack Bartel     bartel@osuosl.org                           |
// +------------------------------------------------------------------------+
// $Id: RangeZoneDataList.php 4074 2006-01-05 21:55:55Z bartel $

/**
 *  RangeZoneDataList.php - contains the {@link RangeZoneDataList} class
 *
 *  @author $Author:$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

/**
 *  This class is used to map Ranges to Zones as opposed to it's cousin
 *  ZoneRangeDataList which is used to map Zones to Ranges. i.e. Lists
 *  all ranges that are associated with a zone.
 *
 *
 *
 */

class RangeZoneDataList extends DB_DataObjectDataList
{

//{{{   vars

    var $_default_rows_per_page = 38;

//}}}
//{{{   get_data_source()

    /**
     *  This method sets up the data source for the data list to be displayed.
     *  In Maintain's case, this means creating DBDataObject Data Source objects.
     */
    function get_data_source()
    {
        global $conf;
        
        $zn = new Zone();
        $zr = DB_DataObject::Factory('zone_range');
        //$zr->joinAdd($zn);
        $src = &new DB_DataObjectDataListSource($zr);
        $this->set_data_source($src);
        $this->_default_orderby = 'id';
    }

//}}}
//{{{   user_setup()

    /**
     *  This method sets up the table columns and various other attributes like
     *  which of those columns is sortable, searchable, etc...
     */
    function user_setup()
    {
        global $conf;
        
        $this->set_actionbar(true);
        $access = true;
        $this->set_form_method(POST);

        $this->add_action_column('checkbox', 'FIRST', 'id');
        $this->set_checkboxvar('zrmaps');

        $this->set_save_vars(array('id'=>$_REQUEST['id']));

        $this->add_header_item(_("Range"), '20%');
        $this->add_header_item(_("Default Domain"), '20%');
        $this->add_header_item(_("Status"), '20%', 'enabled');
        $this->add_header_item(_("Action"), '20%');
        $this->_datasource->setup_db_options(array('zone' => $_REQUEST['id']));
    }

//}}}
//{{{   build_column_item()

    /**
     *  This method takes care of how the data in the columns will appear.
     *
     *  @param  array   $row_data   the row in question
     *  @param  string  $col_name   the column in question
     *
     *  @return Object  $obj        the custom rendering of the data
     */
    function build_column_item($row_data, $col_name)
    {
        switch ($col_name) {
        
        case _('Range'):
            $ran = new Range($row_data['range']);
            $obj = $ran->getDisplay();
            break;
        
        case _('Default Domain'):
            $dom = new Domain($row_data['default_domain']);
            $obj = $dom->getName();
            break;
        
        case _('Status'):
            $obj = ($row_data['enabled'] == 1 ? _("Enabled") : _("Disabled"));
            break;
        case _('Action'):
            
            $del_text = _('Delete');
            $del_path = web_root()."zone.php?action=delete_ranges&amp;zrid="
                        .$row_data['id'];
            $del_link = html_a($del_path,$del_text);

            $endis_text = ($row_data['enabled'] ? _("Disable") : _("Enable")); 
            $endis_path = web_root()."zone.php?action=zone_range_endis&amp;id=".$_REQUEST['id']."&amp;zrid=".$row_data['id']; 
            $endis_link = html_a($endis_path, $endis_text);

            $obj = container($del_link,"|", $endis_link);
            break;        

        default:
            $obj = DefaultGUIDataList::build_column_item($row_data, $col_name);
            
        }

        return $obj;
    }

//}}}
//{{{   actionbar_cell()

    /**
     *  This method is what sets the information display at the bottom of the table.
     *  
     *  @return Container   A span that has strings for displaying the page numbers
     */
    function actionbar_cell()
    {
        $container = container();
        $del_button = html_input('submit', 'action', _('Delete Ranges'));
        $def_button = html_input('submit', 'action', _('Toggle State'));
        $container->add($del_button);

        return $container;
    }
//}}}

}

?>
