<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: CnameCreateForm.php 4606 2006-07-12 22:41:37Z morgabra $

/**
 *  CnameCreateForm.php - Contains the {@link CnameCreateForm} class
 *
 *  @author $Author:$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/MaintainForm.php");

/**
 *  This class is the form that users utilize to create CNAMEs for their current
 *  zone.
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class CnameCreateForm extends MaintainForm
{

/*Vars*/
//{{{   vars

    /**
     *  This array maps form labels to dbdo vars
     */
    var $map = array("Name"=>"name",
                    "Domain"=>"domain",
                    "Server_Name"=>"server",
                    "TTL"=>"ttl");
//}}}
        
/*Methods*/        
//{{{   form_init_elements()

    /**
     *  This method initializes all of the form elements that are going to be
     *  used.
     */
    function form_init_elements()
    {

        $name = new FEText(_("Name"), false);
        $check  = new FECheckbox(_("Entire Domain"));
        $domains = new FEListBox(_("Domain"),false);
        $server = new FEText(_("Server Name"),true);
        $ttl = new FEText(_("TTL"),true);

        //populate the listbox
        $domains->set_list_data($this->_domains());

        $this->add_elements($name, $check,$domains,$server,$ttl);

    }

//}}}
//{{{   form_init_data()
    
    /**
     *  This method sets initial values for form elements if needed
     */
    function form_init_data()
    {
        global $conf;

        $this->set_element_value(_("TTL"),$conf['soa']['default_cname_ttl']);
    }

//}}}
//{{{   form_backend_validation()

    /**
     *  This method validates the data to make sure everything is peachy before
     *  it gets processed and sent into a database or something.
     *
     *  @return Boolean     True if data is valid
     */
    function form_backend_validation()
    {
        $res = true;
        $vals = $this->get_all_element_values();    

        $cnm = new Cname();
        $cnm->setAll();

        if( !($vals[_("Name")] || $vals[_("Entire_Domain")] == 1) ) {
            $res = false;
            $this->add_error(_("Name"),
                        _("Must either use a name or specify the entire domain"));
            $this->add_error(_("Entire Domain"), 
                        _("Must either use name or specify the entire domain"));

        } else {

            //use the form values remapped to their dbdo vars
            $cnm->setFrom($this->remap($vals,DBDO));
            $cnm->zone = $_SESSION['zone'];            

            if($vals[_('Entire_Domain')]) {

                $dom = new Domain($vals[_("Domain")]);
                
                //Make sure it is indeed a subdomain
                if(!$dom->master_domain) {
                    $res = false;
                    $this->add_error(_("Domain"),
                        _("Domain's master domain is not in Maintain."));
                }
                //the domain is the master
                $cnm->setDomain($dom->master_domain);
                //the name is the subdomain
                $cnm->setName(strtok($dom->name,'.'));
                
            }

            $valid = $cnm->validate(); //see if the cname is valid in theory
            

            //If validation failed
            if($valid !== true) {
                //Remap them to form elements
                $errors = $this->remap($valid, FORM);
                
                foreach($errors as $label=>$error) {
                    
                    //if this form element didn't work
                    if($error !== true)
                        $this->add_error($label,$error);
                }
                
                $res = false;
                
            }

        }
        return $res;

    }

//}}}
//{{{   form_action()

    /**
     *  This method preforms actions on the data after validation has been successful
     *
     *  @return Boolean     True if action was successful
     */
    function form_action() 
    {
        global $user;
    
        $res = true;
        $vals = $this->get_all_element_values();

        switch(strtolower($this->get_action())) {

        case 'submit':
            $cnm = new Cname();
            $cnm->query("BEGIN"); //Starts the transaction
            $cnm->setAll();
            $cnm->setFrom($this->remap($vals,DBDO));
            $cnm->zone = $_SESSION['zone'];
            
            if($vals[_('Entire_Domain')]) {

                $dom = new Domain($vals[_("Domain")]);
                
                //Make sure it is indeed a subdomain
                if(!$dom->master_domain) {
                    $res = false;
                    $this->add_error(_("Domain"),
                        _("Domain's master domain is not in Maintain."));
                }
                //the domain is the master
                $cnm->setDomain($dom->master_domain);
                //the name is the subdomain
                $cnm->setName(strtok($dom->name,'.'));
                
            }           
            if(!$cnm->insert()) {
                $cnm->query("ROLLBACK"); //If there was a problem, rollback all ch                                         //changes
                $this->add_error(_("ERROR"), _("Could not add CNAME to the database"));
                $res = false;
            } else {
                $cnm->query("COMMIT"); //Commits the write if everything worked
                $cnm->write_history(_("Created new CNAME"), ACTION_CREATE, $user->username, $_SESSION['zone']);
            }
      
            break;

        default:
            die_hard(_("Invalid form action"));  //how did you get here!?
            break;

        }

        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  This method displays a success message if the action worked
     *
     *  @return DIVTag  $div    Div with the success message in it
     */
    function form_success()
    {
        $div = html_div("msuccess");
        $indlink = html_a(web_root()."index.php", _("[Return to Index]"));
        $cnmlink = html_a(web_root()."cname.php",_('[Return to CNAME List]'));
        $div->add(_("CNAME successfully created!"),html_br());
        $div->add($indlink, $cnmlink);

        return $div;
    }

//}}}
//{{{   _domains()

    /**
     *  This is a helper method that is designed to populate a list box with
     *  domains.
     *
     *  @return Array   $domains    An array of domain ids indexed by name
     */
    function _domains()
    {
        global $user;
        $domains = array();
        
        $zd = DB_DataObject::Factory('zone_domain');
        $dom = DB_DataObject::Factory('domain');
        $dom->joinAdd($zd);
        $dom->orderBy("name ASC");
       
        if(!$user->is_admin())
            $dom->zone = $_SESSION['zone'];
        $dom->query("BEGIN"); //Begins read transaction
        $dom->find();

        while($dom->fetch()) 
            $domains[$dom->name] = $dom->domain;
        $dom->query("COMMIT"); //Commits the read
        return $domains;
    }

//}}}

}

?>
