<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Brad Morgan <morgabra@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: $


require_once(FORMDIR."/MaintainForm.php");

class HostBatchMoveForm extends MaintainForm
{

var $errors = array();
//{{{   form_init_elements()

    /**
     *  This method intializes the fields for the user
     */
    function form_init_elements()
    {  
        global $user;
        
        //Our array ov valid domains and ranges
        $rangeary  = array();
        $domainary = array();
   
        //This will find all the available ranges. It finds all enabled 
        //dynamic ranges that are registered to the current zone. 
        $ranges = DB_DataObject::Factory('ranges');
        $ranges->type = 'dynamic';
        $ranges->enabled = 1;
        $zone_range = DB_DataObject::Factory('zone_range');
        $zone_range->zone = $_SESSION['zone'];
        
        $ranges->joinAdd($zone_range);
            
        $ranges->find();
        
        //after we find all the ranges, we add them to the array.
        while($ranges->fetch()) {
            $rangeary[$ranges->name. ' ' . long2ip($ranges->start) . ' - ' . 
                                    long2ip($ranges->end)] = $ranges->id;
        }

        //This will find all the available domains. It will find any domains that
        //are registered to the current zone.
        $domain = DB_DataObject::Factory('domain');
        $domain->enabled = 1;
        
        $zone_domain = DB_DataObject::Factory('zone_domain');
        $zone_domain->zone = $_SESSION['zone'];
       
        $domain->joinAdd($zone_domain);
       
        $domain->find();
        
        //and after it finds them all, it will add them to an array.
        while($domain->fetch()) {
            $domainary[$domain->name] = $domain->domain;
        }
    
        //the list of ranges are put into a drop-down box
        $rangelist = new FEListBox(_("Target Range"));
        $rangelist->set_list_data($rangeary);
        
        //the list of domains are put into a drop-down box
        $domainlist = new FEListBox(_("Target Domain"));
        $domainlist->set_list_data($domainary);

        //and this is a dirty little hack...
        //When the list of hosts is posted back, either after a form error
        //or whatnot, the hostlist is re-serialized. Unfortunately, upon
        //and error, the hostlist get posted twice, which means it gets 
        //serialized twice. This is bad, because the usual 1 time unserialize
        //will not work. So, this simply takes the hostlist, and will 
        //continue to unserialize it until its an array.

        //yes, its a litte dirty, but its the only thing that works :)
        $var = $_REQUEST['hostlist'];

        while(!is_array($var)) {
            $var = unserialize(stripslashes($var));
        }

        //and this just adds all our new elements to the form.
        $this->add_element(new FEHidden("action", "host batch move"));
        $this->add_element(new FEHidden("hostlist", 
                                           serialize($var)));

        $this->add_element($rangelist);
        $this->add_element($domainlist);
    
    }
//}}}
//{{{   form()

    /**
     *  This method lays the form out for display.
     *
     *  @return DIVTag  $div    Div that contains the form
     */
    function form()
    {   
        $div = html_div("form");
        $cont = container();
      
        //the list of hosts that you checked. If the list isn't an array, then
        //it will unserialize it for you.
        $hostlist = $_REQUEST['hostlist'];
        if(!is_array($hostlist))
            $hostlist = unserialize(stripslashes($_REQUEST['hostlist']));


        //All this stuff is just populating the form with information, nothing
        //is actually done yet!!
        $cont->add(_("You are going to move the following hosts: "), html_br());
        if(!isset($hostlist))
            $cont->add(_("No hosts were selected!"));
        else {
            foreach($hostlist as $hostid) {
                $host = new Host($hostid);
                $cont->add($host->getHostname() . ' ' . $host->getIp(), html_br());
            }
        }

        $cont->add(html_br(), (_("To the Dynamic Range: ")));
        $cont->add($this->element_form(_('Target Range')), html_br());
        $cont->add(html_br(), (_("Domain: ")));
        $cont->add($this->element_form(_('Target Domain')), html_br());
        
        $cont->add(html_br(), (_("*Note: Moved Hosts will be disabled. Remember to re-enable hosts after moving.")));
        
        $cont->add(html_br(), $this->add_action(_("Confirm")), $this->add_action(_("Cancel")));
        $cont->add($this->element_form("action"));
        $cont->add($this->element_form("hostlist"));
        $div->add($cont);

        return $div;
    }

//}}}
//{{{   form_action

    /**
     *  This method defines the behavior for the form based on the button pressed
     *
     *  @return boolean     true if action was successful
     */
    function form_action()
    {
        global $user;
    
        switch(strtolower($this->get_action()))
        {
        case 'confirm':
        
        //The list of hosts that you checked in the beginning
        $hosts = $_REQUEST['hostlist'];
        
        //if $hosts is not an array, then it must be serialized, and so
        //this takes care of that.
        if(!is_array($hosts))
            $hosts = unserialize(stripslashes($_REQUEST['hostlist']));

        //counts how many hosts were selected
        $count = count($hosts);
        
        //The range getting moved to
        $targetrange = $this->get_element_value(_("Target Range"));
        $range = new Range($targetrange);
        
        //The domain getting moved to
        $targetdomain = $this->get_element_value(_("Target Domain"));
        $domain = new Domain($targetdomain);
        
        //Checks to see if there are enough free adresses in the target range.
        if($range->count_free_addresses() >= $count) {
            
            //If there are enough free adresses, then we can begin the transaction
            $host = new Host();
            $host->query('BEGIN');
            foreach($hosts as $hostid) {
        
                $host = new Host($hostid);  //gets the host from its id
                
                //gets the hosts current domain (for record keeping)
                $olddomain = $host->getDomain();
                
                //gets the hosts current ip or range (for record keeping)
                if($host->ip) {
                    $oldip = $host->getIp();
                }
                if($host->dynamic_range) {
                    $oldip = $host->getDynamic_range();
                }
                
                //all range changes will disable the zone and change the domain
                //regardless if the range is actually changed or not.
                $host->setEnabled(0);
                $host->setDomain($targetdomain);
               
                //checks to see if the the hosts current range is the same as
                //the target range. If so, it doesn't bother changing the
                //range again.
                if($host->dynamic_range != $targetrange) {
                    $host->setDynamic_range($targetrange);
                }
                
                //because we are moving, we update the host info in the 
                //database. If moving fails on any host, we rollback
                //everything and add and error.
                if(!$host->update()) {
                    $host->query('ROLLBACK');
                    $errors[$host->id] = _("Move failed for host ID: ").
                                           $host->id;
                }
                    
                //otherwise life is magic and we write to the histroy
                //our recent deeds.
                else {
                    $host->write_history(_("Copied Host from").' '.$oldip.' '.
                    _("to").' '.$range->getDisplay(), ACTION_MODIFY, 
                    $user->username, $host->zone);

                    $host->write_history(_("Changed domain from").' '.$olddomain.
                    ' '._("to").' '.$host->getDomain(), ACTION_MODIFY, 
                    $user->username, $host->zone);

                    $host->write_history(_("Disabled Host"), ACTION_MODIFY, 
                           $user->username, $host->zone);
                }
            }
            //if we have errors in our error array, we display that to the user.
            if(!empty($errors)) {
                foreach($errors as $hostid=>$error) {
                    $this->add_error("Host $hostid", $error);
                }   
                return false;
            }
            
            //if we get here, we win... everything is fine, everything worked,
            //and we can rejoice. So we commit our database changes, and return
            //true.
            else {
                $host->query('COMMIT');
                return true;
            }
        }
        break;
        
        //if the user cancels, it spits them back out to the host datalist.
        case 'cancel':
            header("Location: host.php");
        break;
        
        //if you dont confirm or cancel, you are a bad, bad man and trying to 
        //do something illegal, so we say no way.
        default:
            die_hard(_("Invalid form action"));
        
        }
    }

//}}}
//{{{ form_backend_validation()
    function form_backend_validation()
    {
        return true;
    }
//}}}
//{{{ form_success()
    function form_success()
    {
        //simple success page.
        $div = html_div("success");
        $indxlink = html_a("index.php", "[Return to Home]");
        $hostlink = html_a("host.php", "[Return to Host List]");
        $div->add(container("Hosts moved successfully!", html_br(),html_br()));
        $div->add(container("*Note: Moved Hosts will be disabled! Remember to re-enable moved hosts!", html_br()));
        $div->add($indxlink, $hostlink);
        return $div;
    }
//}}}

}

