<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/tags/release-3.0.0-RC1/class/ip.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |       
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: ip.php 4638 2006-07-21 22:35:46Z schonstal $

/**
  * ip.php - contains the {@link IP} class
  *
  * @author $Author: schonstal $
  * @filesource
  *
  * @package    classes
  */

/** test */
require_once 'Net/IPv4.php';

/**
  * Represents an single IPv4 address
  *
  * This class contains several static functions as well
  *
  * @package    classes
  */
class IP 
{
    // {{{ vars
    
    /** Integer representation of an IPv4 address */
	var $ip_int;

    /** Dotted quad representation of an IPv4 address */
	var $ip_addr;
    
    /** Saves if the IPv4 address was sent in as a quad or an int */
    var $origin;
    
    // }}}

    // {{{ constructor

    /**
      * Constructor
      *
      * @param  mixed   dotted quad (string) or IP integer
      */
	function IP ( $ip_int ) 
    {
		global $conf;

        // if we get passed a dotted quad, convert it to an IP int first
        if (Net_IPv4::validateIP($ip_int))
        {
            $this->origin = "quad";
            //set the address to the passed in dotted quad
            $addr = $ip_int;
            //and create the int version
            $ip_int = Net_IPv4::ip2double($ip_int);
        } else {
            $this->origin = "other";
            // get the dotted quad
            $addr = IP::int2ip($ip_int);
        }
        
        // if it's not valid, return an error
        if (!Net_IPv4::validateIP($addr) ||
            $ip_int != Net_IPv4::ip2double($addr))
            return null;

        // set the internal values
        $this->ip_int   = $ip_int;
        $this->ip_addr  = $addr;

        //var_dump($this);

	} // Host

    // }}}
    // {{{ getSubnet()

    /**
      * Get the Subnet that contains this IP address from the DB
      *
      * @return object  Subnet object or null if no subnet found
      */
	function getSubnet() {
		global $conf;

        // "SELECT * FROM subnet WHERE subnet <= '$this->ip_int' ORDER BY subnet DESC LIMIT 1"
        $dbdo = DB_DataObject::Factory('subnet');
        $dbdo->whereAdd("subnet<='{$this->ip_int}'");
        $dbdo->orderBy('subnet DESC');
        $dbdo->limit(1);
        if ($dbdo->find(true))
            return new Subnet($dbdo->id);
        else
            return null;

	} // get_info

    // }}}

    // {{{ int2ip()

    /**
      * Convert an integer representation of an IP to dotted quad
      *
      * @param  int IP address in integer form
      */
	function int2ip($i) 
    {
		// FIX FOR PHP 4.1
		// http://php.oregonstate.edu/manual/en/function.long2ip.php
		settype($i, "double");

		return long2ip($i);
	} // int2ip

    // }}}
    // {{{ is_free()

    /**
      * Is specified IP address free?
      *
      * @static
      * @param  int     integer representation of an IP address
      * @return boolean true if IP address is not in use by a Host
      */
	function is_free($ip_int) 
    {

        // "SELECT ip FROM host WHERE ip='$ip_int'"
        $dbdo = DB_DataObject::Factory('host');
        $dbdo->selectAdd();
        $dbdo->selectAdd('ip');
        $dbdo->ip = $ip_int;

        if ($dbdo->find())
            return false;
		
		return true;

	} // is_free

    // }}}
    // {{{ ip_exists()

    /**
      * Verify that given IP is contained in an IP Block in the DB
      *
      * @param  int     integer representation of an IP address or
      *                 dotted quad
      * @return boolean true if ip address is contained in an IP block
      */
	function ip_exists($checkip)
    {
        if (!is_numeric($checkip)) {
            // maybe dotted quad, fail otherwise
            if (IP::is_dotted_quad($checkip)) {
                $ip_int = Net_IPv4::ip2double($checkip);
            } else {
                return false;
            }
            
        } else {
            $ip_int = $checkip;
        }
    
        // "SELECT * FROM ip_blocks WHERE start<='$ip_int' AND end>='$ip_int' AND enabled='1'"
        $dbdo = DB_DataObject::Factory('ip_blocks');
        $dbdo->enabled = 1;
        $dbdo->whereAdd("start<='$ip_int'");
        $dbdo->whereAdd("end>='$ip_int'");

        if ($dbdo->find())
            return true;

		return false;

	} //ip_exists

    // }}}
    // {{{ ip2int() (deprecated)

	/**
	*
	* Takes an ip in dotted quad form and converts it to its integer 
    * representation.
    *
    * This is just a wrapper for a Net_IPv4 function, as most of this class is.
	*
	* @access	public
	* @param	$ip		the ip in dotted quad form
	* @deprecated		Please use Net_IPv4::ip2int
	*/
	function ip2int($ip) {
		return Net_IPv4::ip2double($ip);
	} // ip2int

    // }}}
    // {{{ is_dotted_quad() (deprecated)

	/** 
      * Check to make sure it's in dotted-quad format 
      *
      * @static
      * @return boolean true of dotted quad ip string
      */
	function is_dotted_quad($ipaddr) 
    {
        return Net_IPv4::validateIP($ipaddr);
	} //is_dotted_quad

    // }}}

}

?>
