<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: modules.php 4656 2006-07-25 16:49:01Z claym $

/**
 *  modules.php - This file is the module control center
 *
 *  @author     $Author:$
 *  @package    htdocs
 *  @filesource
 */

require_once("includes.php");

// start the session and do some configuring
session_start();
setup_user_environment();

// setup the skin
$skin = Skin::Factory($user->color_template);

// setup the container (page content) object
$content = container();

// if we're using a module:
$info = parse_uri();

switch (count($info)) {

case 2:
    $modname    = array_shift($info);
    $script     = array_shift($info);
    $modpath    = MODDIR . "/$modname/$script";
    $success    = include $modpath;
    if ($success) {
        exit;
    }
    $content->add("$modpath " . _("does not exist"));
    break;

case 1:
    $content->add(_("Please include the script to run in the URI"));
    break;

default:

    // there shouldn't be more than two uri-embedded vars: modname and script
    if (count($info) > 2) {
        $content->add(_("Invalid URI"));
        break;
    }

   // only admins can use this part of the page
    if(!$user->is_admin()) {
        header("Location: denied.php");
    }

    // we'll need the proper datalist for modules
    require_once(HTCLASS."/datalist/ModuleDataList.php");

    // figure out what action we're doing on the list
    switch(strtolower($_REQUEST['action'])) {

    // toggle enabled/disabled status of a module
    case 'enable':
    case 'disable':
        if(!$user->is_admin())
            header("Location: denied.php");
        $errors = array();
        $modarray = $_REQUEST['modules'];
        $act = $_REQUEST['action'];
        
        //invert the state based on the action
        $newstate = (strcasecmp($act,'enable') ? 0 : 1);
        
        //iterate through the checked modules and update
        foreach($modarray as $mod) {
            // call the enable or disable function, respectively
            // (same as: Module::en/disable(...))
            $success = call_user_func_array(array('module', $act), array($mod));
            if ($success !== true) {//log errors
                if (PEAR::isError($success)) {
                    $errors[$mod] = $success->getMessage();
                } else {
                    $errors[$mod] = _("Module state could not be changed!");
                }
            }
        }
        
        //if there were errors
        if(!empty($errors)) {

            $div = html_div("errors");
            $table = html_table();
            $table->add_row(_("The following errors occured"));
            foreach($errors as $mod=>$error) {
                $table->add_row($mod,$error); //add error
            }
            $div->add($table);
            $content->add($div); //add error block
        }
        
    // display the modules and their statuses
    case 'show':
    default:
        if(!$user->is_admin())
            header("Location: denied.php");
        $title = html_span("title");
        if(isset($_REQUEST['name'])) {
            $title->add(_("Displaying Module ").$_REQUEST['name']);
            $module = DB_DataObject::Factory('modules');
            $module->name = $_REQUEST['name'];
            $module->find(1);
            $content->add($title);
            $content = Module::apply_filter("object_info",$content,$module);
        } else {
            $title->add(_("Showing Modules"));
            $content->add($title);
            $list = new ModuleDataList(_("Modules"));
            $content->add($list);
        }

    } // switch $_REQUEST['action']

} //switch URI args

// echo the page content
print $skin->display($content);

?>
