<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/tags/release-3.0.0-RC1/includes/lib.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: lib.php 4708 2006-08-08 22:18:03Z wenzel $

/**
  * Supporting functions for Maintain
  *
  * @package    includes
  */

// {{{ setup_user_environment()

/**
  *
  * Setup the users "environment" upon any page load.  It is intended to be
  * run after session_start() and before any work actually happens
  *
  */
function setup_user_environment() 
{
    global $conf, $user, $images;

    // first thing's first, check for the gone fishing file
    if (file_exists($conf['paths']['prefix'] . "/build/gone.fishing") ) {
        show_template('gonefishing');
        exit();
    }

    // First create the user object from $_SESSION['user']

    // if we haven't set up the user, or it expired, start over
    if (!isset($_SESSION['_authsession']['username'])) {
        header("Location: ".web_root().'logout.php');
        exit();
    }

    // otherwise, instantiate the user
    $user = new User($_SESSION['_authsession']['username']);

    /* choose a zone here */
    if (isset($_POST['zoneselect']) && $user->get_access_level($_POST['zoneselect'])) {
        // first, if we're specifying a zone, switch to it
        $_SESSION['zone'] = $_POST['zoneselect'];
        
    } elseif (isset($_SESSION['zone'])) {
        // or, if there's a zone stored in the SESSION, just keep it
        // (i.e. do nothing)

    } else {
        // if nothing else, we try the user's preferred zone
        $_SESSION['zone'] = $user->preferred_zone;
    }

    // put the user in this zone and set permissions accordingly
    $user->setZone($_SESSION['zone'], true);

    $conf['action'] = $_REQUEST['action'];
    $conf['script'] = $SCRIPT_NAME;

    // if they don't specify an action we want to clear the user view
    //    so it doesn't inherit the current view for a different list.
    if (!$_REQUEST['action'])
        $user->view  = false;

    // we have established this person as a real user
    $conf['general']['max_list'] = $user->list_view;

    // alright, let's make sure the session is set up right for next time
    $_SESSION['_authsession']['username']   = $user->username;
    //If they have canceled something then redirect off to a template
    if ( $action == 'Cancel' ) {
        require $conf['paths']['prefix'] . "/templates/show_cancel.inc";
        exit();
    } //cancel

}

// }}}

// {{{ dbh()

/**
*
* Returns the global Maintain database handle
*
* Also does some error handling if the dbh is no longer a resource.
*
* @return    resource the global database connection for maintain
*/
function dbh() 
{
    global $conf;

    static $dbh;

    // if we're already set up, go!
    if (is_resource($dbh))
        return $dbh;

    // just cuz $conf['database']['something'] is too long to use 3 times as args
    $dbinfo = $conf['database'];

    // make the db handler and select the proper database
    $dbh = mysql_connect($dbinfo['host'], $dbinfo['username'], $dbinfo['password']);
    $success = mysql_select_db($dbinfo['database'], $dbh);

    if(!is_resource($dbh) || !$success) {
        $dbh = null;
        die("Fatal Error: Database Connection Failed. Please check your Maintain database.");
    }

    return $dbh;
    
} // dbh

// }}}

// {{{ get_available_ranges()

/**
  * Get a list of potential IP Ranges not in use
  *
  * Note: This function is actually not all that slow anymore
  * and seems to be perfectly functional
  *
  * @return array   list of objects containing:
  *                 $start (ip int), $end (ip int), and $total (int)
  */
function get_available_ranges()
{

    // set up the return array
    $ranges = array();

    /* 1) Step through each IP Block */
    $ipblock = DB_DataObject::Factory('ip_blocks');
    $ipblock->enabled = 1;
    $ipblock->find();

    // if no results were found, return the empty array
    if (!$ipblock->fetch()) {
        return $ranges;
    }

    /* 2) Iterate through each IP Block, looking for gaps in IP Ranges >= 2 */

    do {
        // set up the first start/end range
        $start  = $ipblock->start;

        // find the first IP Range in this IP Block
        $range = DB_DataObject::Factory('ranges');
        $range->whereAdd("start>='$start'");
        $range->whereAdd("end<='$ipblock->end'");
        if (!$range->find()) {
            break;
        }

        // step through each Range looking for gaps
        while ($range->fetch()) {
            
            // get the end of this unused block of IPs (from the IP Block)
            $end = $range->start - 1;

            /* 3) if there were >2 unused IPs, this is a free range */
            $total = $end - $start;
            if ($end - $start > 2) {
                $unused->start  = $start;
                $unused->end    = $end;
                $unused->total  = $total;
                $ranges[]       = $unused;
            }

            // set the new starting address
            $start = $range->end + 1;

        } //each range

    } while ($ipblock->fetch());

    return $ranges;

}

// }}}
// {{{ get_available_subnets()

/**
  * Get a list of all available subnets
  *
  * @return array   list of objects containing:
  *                 $start (ip int) and $netmask (ip dotted quad)
  */
function get_available_subnets () 
{

    // set up the return variable
    $subnets = array();

    /* Get a list of all enabled IP Blocks */
    $ipblock = DB_DataObject::Factory('ip_blocks');
    $ipblock->enabled = 1;

    // if no enabled IP Blocks exist, get out
    if (!$ipblock->find()) {
        return $subnets;
    }

    // blah;
    return $subnets;
}
/*
        // just like get_available_ranges, the old version
        // was horribly inefficient, and broken
        // so it's been rewritten

        global $conf;

        //Get IP Blocks
        $dbdo = DB_DataObject::Factory('ip_blocks');
        $dbdo->enabled = 1;
        $dbdo->find();

        while ($dbdo->fetch())
            $ip_blocks[] = $dbdo;

        $ip = new IP(0);

        //Make this here we will need it for later
        $max_netmask = $ip->ip2int('255.255.255.255');

        //go through all ip blocks
        foreach ($ip_blocks as $ip_block) {

            $dbdo = DB_DataObject::Factory('subnet');
            $dbdo->whereAdd("start>='{$ip_block->start}' AND end<='{$ip_block->end}'");
            $dbdo->find();

            $subnet_start = $ip_block->start;
            while ($dbdo->fetch()) {

                if(substr($ip->int2ip($dbdo->start),-2) === '.2')
                    $subnet_end = $dbdo->start - 3;
                else
                    $subnet_end = $dbdo->start - 2;

                //if they has been at least 1 empty IP
                //create a new range object manualy
                if ($subnet_end - $subnet_start > 1) {
                    //we add 1 so it includes each end.. 1 to 5 is 5 possible not 4
                    $subnet->start     = $ip->int2ip($subnet_start);
                    $diff = $subnet_end - ( $subnet_start + 1 );
        
                    $netmask = $max_netmask - $diff;

                    //now current get the one right before
                    $subnet->netmask = $ip->int2ip($netmask);

                    $subnets[] = $subnet;

                } //end start != end

                $subnet_start = $dbdo->end + 1;

            } //end while inside ip_block

        } //foreach ip_blocks

        return $subnets;

} // get_available_subnets
*/

// }}}

// {{{ split_hostname()

/**
  *
  * Parse out the various bits of a hostname
  *
  * @param  string  FQDN
  * @return object  Returns an object with the following variables:
  *                     - name, the 'hostname' ex. abe.osuosl.org, name == abe
  *                     - domain, the full domain ex. osuosl.org
  *                     - tld, top level domain ex. org
  */
function split_hostname($hostname) 
{

    // this is the position of the first '.' in the FQDN
    // effectively, it separates the name from the domain
    $pivot = strpos($hostname, '.');

    // if it's not a FQDN, return the hostname unadultered
    if (!$pivot) {
        $object->name = $hostname;
        $object->domain = '';
        $object->tld = '';
        return $object;
    }

    // break off the name from the FQDN
    $object->name = substr($hostname, 0, $pivot);

    // break off the domain
    $object->domain = substr($hostname, $pivot + 1);

    // Figure out what the top level domain is
    $object->tld = substr($hostname, strrpos($hostname, '.') + 1);

    //we should be done return the object 
    return $object;

} //split_hostname

// }}}
// {{{ split_domain()

/**
  * Takes off X levels of the specified domain
  *
  * For example, from cla.orst.edu it would take .edu for level=1
  * 
  * For cla.orst.edu it would take .orst.edu for level=2
  *
  * @param  string  domain
  * @param  int     number of levels to take from domain
  * @return string  last $level number of subdomains
  */
function split_domain($domain,$level) 
{
    
    // Due to how split works decrease it by one
    // On john.cla.orst.edu value of 1 gives you orst.edu 
    // while in domain land it should give you only edu
    $level = $level - 1;

    // Get preiod count so we can split on 
    // count - level periods
    $count = substr_count($domain,".") + 1;

    $data = split("\.",$domain,$count-$level);
    
    // Destroy the last bit, thats the part we don't care about
    array_pop($data);
    $data = implode(".",$data);
    
    return $data;
    
} //split_domain

// }}}

// {{{ deliver_csv()

/**
  * Build a CSV file with all of the hosts passed to it
  * and mail it off to the specified user
  *
  * @param  string  email address to send CSV to
  * @param  int     ID of zone used in query
  * @param  array   array of the hosts' IDs (ints)
  * @param  array   array of the cnames' IDs (ints)
  * @param  array   array of the mx records' IDs (ints)
  * @param  array   array of the flo IDs (ints)
  * @return boolean true on successful send, false on error
  */
function deliver_csv ($email, $zone, $hosts=0,$cnames=0,$mxs=0,$flos=0) 
{
    //if the hosts is set send a host report
    if ($hosts) {

        $mail_body  = "# format for this file is the following\n";
        $mail_body .= "hostname,ip,hardware address,operating system,hardware type,location,serial number,host status,bandwidth rate,purchase date,po number,warranty date,owning unit,other id,last seen\r\n";

        foreach ($hosts as $hid) {
            $host       = new Host($hid);
            $domain_obj = new Domain($host->domain);
            $domain     = $domain_obj->getName();
            $date       = $host->getLast_seen();
            $ip_addr    = $host->getIp();
            if (!$ip_addr) {
                $ip_addr = $host->getDynamic_range();
            }

            //we need to do a str_replace for ,'s do they don't break the csv
            $host->os       = str_replace(",","_",$host->os);
            $host->type     = str_replace(",","_",$host->type);
            $host->location = str_replace(",","_",$host->location);
            $host->serial   = str_replace(",","_",$host->serial);
            $host->other_id = str_replace(",","_",$host->other_id);

            $mail_body .= "$host->name.$domain,$ip_addr,$host->ha,$host->os,$host->type,$host->location,$host->serial,$host->status,$host->bandwidth_rate,$host->purchase_date,$host->po_number,$host->warranty_date,$host->owning_unit,$host->other_id,$date\r\n";
        } //foreach host

        mail ($email, "CSV Host Report of $zone", $mail_body, 
                    "From: $email\r\n" .
                    "X-Mailer: NET Mail Gateway PHP/" . phpversion() . "\r\n");

    } //end host report

    //if the cnames are set send a cname report
    if ($cnames) {

        $mail_body  = "# format for this file is the following\n";
        $mail_body .= "# hostname,cname server\n";

        foreach ($cnames as $cname_id) {
                $cname = new Cname($cname_id);
                $mail_body .= "$cname->hostname,$cname->server\n";
        }

        mail ($email, "CSV Cname Report of $zone", $mail_body,
                "From: $email\r\n" .
                "X-Mailer: NET Mail Gateway PHP/" . phpversion());
    } //end cname report

    //if the mxss are set send a mx report
    if ($mxs) {

        $mail_body  = "# format for this file is the following\n";
        $mail_body .= "# hostname,mx server,priority\n";

        foreach ($mxs as $mx_id) {
                $mx = new MX($mx_id);
                $mail_body .= "$mx->hostname,$mx->server,$mx->priority\n";
        }

        mail ($email, "CSV MX Record Report of $zone", $mail_body,
                "From: $email\r\n" .
                "X-Mailer: NET Mail Gateway PHP/" . phpversion());

    } //end mx report

    //if there are flos set send a flo report
    if ($flos) {

        $mail_body  = "# format for this file is the following\n";
        $mail_body .= "date,hostname,domain,os,location,hardware type,serial,other_id,ip,in_kb,out_kb,in_packets,out_packets\n";

        foreach ($flos as $flo) {
            $date = date("m\/d\/Y",$flo->date);
            // get the host object for the Flo module
            $host = DB_DataObject::Factory('host');
            $host->ip = $flo->ip;
            $host->find(true);

            $hostname = $host->name;
            $ip_addr = $host->ip->ip_addr;
            $serial = $host->serial;
            $os = $host->os;
            $location = $host->location;
            $other_id = $host->other_id;

            // get the name of the domain
            $dbdo = DB_DataObject::Factory('domain');
            $dbdo->get($host->domain);
            $domain = $dbdo->name;

            $os             = str_replace(",","_",$os);
            $type             = str_replace(",","_",$type);
            $location         = str_replace(",","_",$location);
            $serial            = str_replace(",","_",$serial);
            $other_id        = str_replace(",","_",$other_id);

            $mail_body .= "$date,$hostname,$domain,$os,$location,$type,$serial,$other_id,$ip_addr,$flo->in_kb,$flo->out_kb,$flo->in_packets,$flo->out_packets\n";
        }

        mail ($email, "CSV Bandwidth Usage Report of $zone", $mail_body,
                "From: $email\r\n" .
                "X-Mailer: NET Mail Gateway PHP/" . phpversion());

        } //end flos report

} //deliver_csv

// }}}

// {{{ commify()

/**
  * Given an integer, returns that number in human-readable form (i.e. with commas)
  *
  * Example: commify(1000000) --> 1,000,000
  *
  * @param  int     the integer to convert
  * @return string  the parameter with commas every three digits
  */
function commify($int) 
{
    $count = intval(strlen($int)/3);
    if ($count == strlen($int)/3) { $count -= 1; }
    $i=1;
    $b=0;
    while ($i <= $count) {
        $int = substr_replace($int, ",",((strlen($int)-$b)-(3 * $i)),0);
        $i++;
        $b++;
    }
    return $int;

}

// }}}

// {{{ setup()

/**
  * Setup the global config array and db_dataobjects
  *
  * @global array   $conf   parsed config file
  */
function setup() 
{
    global $conf;

    // read in the config
    $conf = parse_ini_file(CONFIG, true);
    
    // grab the right phphtmllib folder from the configs
    $phphtmllib = $conf['paths']['phphtmllib'];
    if ($phphtmllib{0} != '/') $phphtmllib = MAINTAIN_ROOT.'/'.$phphtmllib;
    define("PHPHTMLLIB", realpath($phphtmllib));

    // set up db_dataobjects
    configure_db_dataobject();
}

// }}}
// {{{ configure_db_dataobject()

/**
 * Sets up DB_DataObjects
 *
 * @access  private
 * @global  mixed   $conf   Maintain global config
 * @return  array   $array  An altered used as options for a DB Data Object
 */
function configure_db_dataobject()
{
    global $conf;

    DB_DataObject::debugLevel(5);

    // just as shortcuts
    $db = $conf['database'];
    $path = $conf['paths']['prefix'].DAODIR;
   
    // generate the DSN based on config options
    $dsn = 'mysql://' . $db['username'] . ':' . $db['password'];
    $dsn .= '@'. $db['host'] . '/' . $db['database'];

    // set the db_dataobject options
    /*
    NOTE: extends_location is an ugly hack. The DB_DataObject Generator
          will put it into single quotes like:
          require_once '$opts[extends_location]';
          So in order to put the constant DAODIR in there, we close the
          single quote immediately, add DAODIR, and re-open the single
          quotes afterwards. This results into:
          require_once ''.DAODIR.'/Maintain.php';
          This allows us to generate machine-independent DAO scripts.
    */
    $opts = array('database'=>$dsn,
                'schema_location'=>DAODIR,
                'class_location'=>DAODIR,
                'require_prefix'=>DAODIR,
                'class_prefix'=>'DAO_',
                'extends'=>'DAO_Maintain',
                'extends_location'=>"'.DAODIR.'/Maintain.php",
                'proxy'=>'full');

    // initialize DB_DataObjects' options from:
    // http://pear.php.net/manual/en/package.database.db-dataobject.intro-configuration.php
    require_once('PEAR.php');
    require_once('DB/DataObject.php');
    $options = &PEAR::getStaticProperty('DB_DataObject','options');
    $options = $opts;

    // return the options array
    return $options;

}

// }}}

// {{{ get_files()

/**
  * Given a path, return a list of all files
  * in that directory
  *
  * @param  string  path
  * @param  boolean true if recursive
  * @param  boolean true if files should be 'require_once'd
  * @return array   list of all files by name (relative paths)
  */
function get_files($path, $recurse = false, $req = false) 
{
    global $phphtmllib;

    $files = array();

    $debug = false;
    $baddirs = array('.', '..', '.svn', 'tests', 'lib', 'bin');
    $validext = array('php');

    if ($debug) {
        echo "get_files() called on:\t$path\n";
        echo "recursion:\t";
        if ($recurse)
            echo "ON\n";
        else
            echo "OFF\n";
    }

    // make sure we're passed a valid path
    if ($handle = opendir($path)) {

        // make sure the path is readable
        while (($file = readdir($handle)) !== false) {

            // for making life easier
            $fullpath = "$path/$file";

            // directory tests
            if (is_dir($fullpath)) {

                // make sure it's not a blacklisted directory
                if (in_array($file, $baddirs))
                    continue;

                // if it's a directory, descend in if '$recurse'
                if ($recurse) {
                    if ($debug)
                        echo "\trecursing:\t$fullpath\n";
                    $files = array_merge($files, get_files($fullpath, $recurse));
                    if ($debug)
                        echo "\trecursion of '$fullpath' complete\n";
                }

            }

            // file tests
            else {

                // determine the file extension
                $ext = pathinfo($fullpath, PATHINFO_EXTENSION);

                // if we've determined to include that type, do so
                if (in_array($ext, $validext)) {
                    if ($debug)
                        echo "\t\tadded:\t$fullpath\n";
                    $files[] = $fullpath;
                }
            }
        } //while files

        closedir($handle);

    } // if valid dir

    // if we're supposed to require these files, do so
    if ($req)
        foreach($files as $file)
            include_once($file);

    return $files;
}

// }}}

// {{{ format_date()

/**
 *  This function is used to format dates in a consistent way across Maintain
 *
 *  @param int  $date   The date in int form
 *  @return string  The formatted date
 */
function format_date($date)
{
    return date("F j, Y, g:i a", $date);
}

// }}}

//{{{   validate_comment()

/**
 *  This method is used to validate the comment fields in all the forms
 *
 *  @param  String  $comment    The comment to be validated
 *  @return Mixed               True if valid, error string otherwise
 */
function validate_comment($comment)
{
    // Regex: Must be a string that must be comprised of one or more 
    //        characters that are either a-z A-Z 0-9 white space or '.
    if(ereg("^[[:alnum:]'[:space:]]+$",$comment))
        return true;
    else
        return _("Comments must be alpha numeric characters only");
}

//}}}

// {{{ web_root()

/**
  * Return the web_path from the config file with a trailing slash
  *
  * @return string  the value of the web_path in the maintain config file
  */
function web_root()
{
    global $conf;

    $web_path = $conf['paths']['web_path'];

    if (substr($web_path, -1) != '/') {
        $web_path .= '/';
    }

    return $web_path;
}

// }}}

// {{{ parse_uri

/**
  * For pages relying on RESTian architectures, this 
  * returns the RESTian info as an array.
  *
  * Example: /modules/autoreg/display.php would return:
  * array (0 => "autoreg", 1 => "display.php")
  *
  * @return array   array of uri path elements
  */
function parse_uri()
{
    $nav = $_SERVER['REQUEST_URI'];
    $script = $_SERVER['SCRIPT_NAME'];
    $nav = ereg_replace("^$script", "", $nav);

    $vars = explode('/', $nav);
    // strip off the first '/'
    array_shift($vars);

    return $vars;
}

// }}}

// {{{ fresh_install()

/**
 * Verifies that maintain has been installed and
 * at least initially configured, otherwise redirects
 * to the install script.
 *
 * If maintain has been installed, the function returns true
 */
function fresh_install()
{
    global $conf;

    // if $conf doesn't exist, we need to run setup
    if (!$conf) {
        setup();
    }

    // get htdocs/install.php for constants and functions
    require_once(INCLUDEDIR . "/install.php");

    // ask install.php if we're installed properly
    // we do it this way to install.php can determine
    // where we're at in the installation process
    // and maintain its own constants, etc.
    $status = install_status();

    if ($status !== true) {
        header("Location: ".web_root()."install.php");
        exit();
    }

    return true;

}

// }}}

// {{{ daos_exist()

    /**
      * A convenience function to determine if 'create_tables'
      * has been run.  All it really does it count the number of 
      * files in the DB_DataObject ['schema_location'] directory
      * and make sure there are more than just the base files around
      */
    function daos_exist()
    {
        // ok, ugly hack, and I apologize, but:
        // we cannot include any extended DAO objects if
        // the base DAOs do not exist.  So, I'm making sure
        // there are at least 4 .php files in this directory before executing this
        // next include.  Why 4?  Maintain.php, maintain.ini, maintain.links
        // will always be present.

        // get a list of files in that directory
        // determine the file extension
        $files = get_files(DAODIR);

        // count the number of php files in this directory
        $scripts = 0;
        foreach ($files as $file) {
            $extension = pathinfo($file, PATHINFO_EXTENSION);
            if ($extension == 'php') {
                $scripts++;
            }
        }
        
        // less than 4 scripts are bad (see above)
        if ($scripts < 4) {
            return false;
        }

        return true;
    }

// }}}
//{{{ show_live_time()
/**
	*
	* Creates a string that tells when a object will be next updated.
	*
	* This functions tests for all cases and will return one of three cases:
	* - Host up to date: "$name is up to date"
	* - Host will update: "This $name will be updated in $minutes_remaining
	*						minutes on $date at $time"
	* - Updates have stopped: "Update have been paused."
	*
	*
	* @return	string	 Description
	* @param	$last_update 	The last time this object was updated 
	*							@see ObjectHistory
	* @param	$name			The type of object being updated (ex. host)
	* @param	$domain_id		The ID of the domain that this object belongs
	*							too, used to find the refresh time, if this is
	*							not passed in the function uses 
	*							$conf['update_time']
	* @access	public
	*/
	function show_live_time($last_update, $name, $domain_id = 0) {
		global $conf;

		// The estimated update time in minutes
		$update_min = $conf['update_time']/60; 

		if ($domain_id) {
        	// If a domain_id was passed try to get the SOA refresh time to use
			// instead of $conf['update_time'] 
        	$sql = "SELECT soa.refresh FROM soa,domain,domain as master_domain where ( domain.master_domain=master_domain.id AND soa.domain=master_domain.id AND domain.id='$domain_id') OR soa.domain='$domain_id' LIMIT 1";
        	$db_results = mysql_query($sql, dbh());
			$results = mysql_fetch_object($db_results);

			if ($results->refresh) { $update_min = $results->refresh/60; }
		}
		
		$time_array = localtime($last_update,1);

        if ($update_min == 0)
            $update_min++;

		$time_array['tm_min'] = (intval($time_array['tm_min']/$update_min)+2) * $update_min;
		$estimated_time = mktime($time_array['tm_hour'],$time_array['tm_min'],$time_array['tm_sec'],$time_array['tm_mon']+1,$time_array['tm_mday'],$time_array['tm_year']+1900,$time_array['tm_isdst']);

		// Remaining number of minutes is	
		$minutes_remaining = intval(($estimated_time - time())/60);
		// Estimated date 
		$date = date('l F d Y', $estimated_time);	
		// Estimated time
		$time = date('h:i a', $estimated_time);

		if ($estimated_time <= time()) {
			return "<strong>$name is up to date</strong>";
		}

		$stop_file = $conf['prefix'] . "/build/stop.update";

		if (@file_exists($stop_file)) { 
			return "<font class=\"error\">Updates have been paused. Please wait 10 minutes and try again. </font>";
		}
		
		if ($minutes_remaining > 1) { 
			$text = "<font class=\"error\">This $name will be updated in $minutes_remaining minutes on $date at $time.</font>";
		}
		else {
			$text = "<font class=\"error\">This $name will be updated in less than a minute.</font>";
		}

		return $text;

	} // show_live_time
	
//}}}

// {{{ die_hard()
/**
  * Maintain should never die() uncontrolledly. However, if it does/has to,
  * this is almost always a bug in the code (unless a script kiddie played
  * with hidden form input values etc). Therefore, people should file a bug
  * if die_hard() ever occurs.
  *
  * @param String optional message
  * @return void (dies)
  */
    function die_hard($msg = '')
    {
        if (strlen($msg) > 0) $msg .= "\n\n";
        die($msg._("Fatal error. If you believe this is a program bug, please file a bug on: ").MAINTAIN_WEBSITE);
    }
// }}}
?>
