<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/tags/release-3.0.0-RC1/includes/session.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <danny@osuosl.org>                            |
// |            Frederic Wenzel <fredosuosl.org>                            |
// +------------------------------------------------------------------------+
// $Id: session.php 4499 2006-06-22 21:14:01Z wenzel $

/**
  * This script handles the session handling for Maintain so that it resides
  * in the DB instead of local files.
  *
  * This also allows us to use DB_DataObjects to manipulate session data
  *
  * @package    includes
  */

// set the necessary PHP variables based on config values
ini_set("session.save_handler", "user");
ini_set("session.name", $conf['session']['name']);
ini_set("session.cookie_secure", $conf['session']['cookie_secure']);
ini_set("session.cookie_domain", $conf['session']['cookie_domain']);

ini_set("session.gc_maxlifetime", $conf['session']['timeout']);
// set the % chance that php will check for stale sessions
ini_set("session.gc_probability", 100);
ini_set("session.gc_divisor", 100);

// force PHP to use the functions below for session management
$result = session_set_save_handler("dbOpen", "dbClose", "dbRead", "dbWrite", "dbDestroy", "dbGarbage");


// {{{ open

/**
  * Open a new session.  Initializes globals, etc.
  *
  * @access private
  * @param  string  $path       session.save_path
  * @param  string  $sessname   session.name
  * @return boolean false on error
  */
function dbOpen($path, $sessname) 
{
    $session = DB_DataObject::Factory('session');
    if (!$session)
        return false;

    return true;
}

// }}}
// {{{ close

/**
  * Close a session. Unsets globals, etc.
  *
  * @access private
  * @return boolean false on error
  */
function dbClose() 
{
    return true;
}

// }}}
// {{{ read

/**
  * Read a session's data based on session id
  *
  * @access private
  * @param  string  $id HEX representation of the session ID
  * @return string  serialized string of session data
  */
function dbRead($id) 
{
    global $conf;

    // open the DB
    $session = DB_DataObject::Factory('session');

    if (!$session || PEAR::isError($session))
        return '';

    $session->sessid = $id;

    // session expired?
    // calculate the expiration time
    $sessionLifetime = $conf['session']['timeout'];
    $expires = time() - $sessionLifetime;

    // return only non-expired sessions
    $session->whereAdd("last_access > '$expires'");

    // fetch the current session
    $result = $session->find(true);

    // if nothing was found, return an empty string
    if (!$result)
        return '';
    else
        return $session->data;
}

// }}}
// {{{ write

/**
  * Writes session data to the database ($_SESSION)
  * or create a session row if none present
  *
  * @access private
  * @param  int     $id     String representation of the hex session id
  * @param  string  $data   Session data to store
  * @return boolean true if data successfully written
  */
function dbWrite($id, $data) 
{
    // open the DB
    $session = DB_DataObject::Factory('session');

    // set the session id
    $session->sessid = $id;

    // check for current session row
    $result = $session->find();

    // set the access time
    $session->last_access = time();
    
    // set the data
    $session->data = $data;

    // if nothing was found for this session, insert a new row
    if (!$result)
        $result = ($session->insert()) ? true : false;
    else
        $result = ($session->update()) ? true : false;

    return $result;
}

// }}}
// {{{ destroy

/**
  * Destroy the active session
  *
  * @access private
  * @param  int     $id     String representation of the hex session id
  * @return boolean true on successful deletion
  */
function dbDestroy($id) 
{
    // open the DB
    $session = DB_DataObject::Factory('session');

    if (!$session || PEAR::isError($session))
        return true;

    // set the condition
    $session->whereAdd("sessid='$id'");

    // delete the row
    $result = $session->delete(DB_DATAOBJECT_WHEREADD_ONLY);

    if (!$result)
        return false;

    return true;

}

// }}}
// {{{ garbage collector

/**
  * Garbage collection function.
  * Periodically run to delete inactive sessions
  *
  * @access private
  * @param  int     $life   session.gc_maxlifetime
  * @return boolean true if no errors occured
  */
function dbGarbage($life) 
{
    // calculate the expiration time
    $expires    = time() - $life;

    // open the DB
    $session = DB_DataObject::Factory('session');

    if (!$session || PEAR::isError($session))
        return true;

    // set the condition
    $session->whereAdd("last_access < '$expires'");

    // delete the rows
    $rows = $session->delete(DB_DATAOBJECT_WHEREADD_ONLY);

    // if 0 or more rows were deleted, it ran successfully
    if (is_numeric($rows))
        return true;

    // otherwise, an error occured
    return false;
}

// }}}

?>
