<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2005 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id$

/**
  * Contains the {@link Build_VMPS} class
  *
  * @author $Author$
  * @filesource
  *
  * @package    classes
  * @subpackage backend
  */

require_once('PEAR.php');
require_once(MODDIR.'/module.php');
require_once(CLASSDIR.'/ip.php');

/**
 * Builds VMPS config files for dynamic VLANs on Cisco switches
 * and pushes them to the respective boxes
 *
 * @package classes
 * @subpackage backend
 */
class Build_VMPS extends Module
{
// {{{ vars
// }}}

/* Methods for Config File Intergrity Checking */
// {{{ needsConfigFile()

    /**
      * Returns, if the module needs a config file at all. This will be used
      * by the automated config file integrity checks, for example.
      * Defaults to false.
      * Can be called statically.
      *
      * @return boolean .cfg file vital for the module to work?
      */
    function needsConfigFile()
    {
        return true;
    }
// }}}

/* Module Overloads */
// {{{ _install()

    function _install()
    {
        return true;
    }

// }}}
// {{{ _uninstall()

    function _uninstall()
    {
        return true;
    }

// }}}
// {{{ initialize()

    function initialize()
    {
        return true;
    }

// }}}
// {{{ register_hooks()
    /**
      * Connect this module to the module hooks it should reply to
      */
    function register_hooks()
    {
        Module::add_action('backend_build', 'buildVMPS');
        Module::add_action('backed_push', 'buildVMPS');
    }
// }}}

/* Private Module Functions */
// {{{ buildVMPS()
/**
 * builds Maintain VLAN configs
 * @return boolean success
 * @access private
 */

    function buildVMPS()
    {
        // what path are we building to?
        $buildfilename = BUILD .'/'. $this->config['build_vmps']['build_file'];
        // header ("boiler") file name
        $boilerfile = $this->_path . '/templates/vmps.boiler';
    
        // make a copy of the old config file that we can diff later
        if (file_exists($buildfilename)) {
            if (!copy($buildfilename, "${buildfilename}.bak"))
                PEAR::raiseError(_('Build_VMPS could not duplicate vmps config file. Check write permissions on build dir!'));
        }

        if (!$fh = @fopen($buildfilename, 'w')) {     // try to open our config file for write access
            PEAR::raiseError(_("Build_VMPS does not have write access to ${buildfilename}!"));
            return false;
        }
        
        // include header file
        if (file_exists($boilerfile)) {
            $this->_include_file($fh, $boilerfile);
        }

        // now fetch all the VLAN hosts and print them out to the config file
        $this->_print_hosts($fh);
        
        fclose($fh);
    }
// }}}
// {{{ pushVMPS()
/**
 * pushes VMPS configs to the cisco boxes
 * @return boolean success
 * @access private
 */

    function pushVMPS()
    {
        // no pushing of VMPS information yet
    }
// }}}

// {{{ _print_hosts()
/**
 * fetch VLAN hosts and print them to the config file
 *
 * @param Filehandler pointing to config file
 * @return void
 * @access private
 */
    function _print_hosts(&$fh)
    {
        // $mode comes from bin/build/all and will tell us if debug is enabled
        global $mode;

        // select hosts and their domains
        $hosts = DB_DataObject::Factory('host');
        $domain = DB_DataObject::Factory('domain');
        $hosts->joinAdd($domain);
        
        $hosts->whereAdd('ha != 0');
        $hosts->enabled = 1;
        $hosts->selectAdd();
        $hosts->selectAdd('DISTINCT host.ha, host.name, host.ip, host.dynamic_range, domain.name AS domainname');
        $hosts->orderBy('ha');
        $hosts->find();
        
        $range_cache = array();     // cache dyn_range->subnet->vlan mappings
        while ($hosts->fetch()) {
            $dotted_ha = substr(chunk_split($hosts->ha, 4, '.'), 0, -1);    // dotted hardware address
            
            if ($hosts->ip == 0) {   // get VLAN for dynamic host
                if (!isset($range_cache[$hosts->dynamic_range])) {         // not cached yet?
                    $range = DB_DataObject::Factory('ranges');
                    $subnet = DB_DataObject::Factory('subnet');
                    $vlan = DB_DataObject::Factory('vlan');
                    $subnet->joinAdd($vlan);
                    $range->joinAdd($subnet);

                    $range->id = $hosts->dynamic_range;
                    $range->selectAdd();
                    $range->selectAdd('vlan.name AS vlanname');
                    $range->find(true);
                    
                    $vlanname = $range->vlanname;
                    $range_cache[$hosts->dynamic_range] = $range->vlanname; // cache it!

                    $vlan->free();
                    $subnet->free();
                    $range->free();
                    
                } else { // get VLAN from cache
                    $vlanname = $range_cache[$hosts->dynamic_range];
                }
                
            } else {                // get VLAN for static host
                $subnet = DB_DataObject::Factory('subnet');
                $vlan = DB_DataObject::Factory('vlan');
                $subnet->joinAdd($vlan);
                $subnet->whereAdd("subnet < {$hosts->ip}");
                $subnet->orderBy('subnet DESC');
                $subnet->limit(1);
                $subnet->selectAdd();
                $subnet->selectAdd('vlan.name AS vlanname');
                $subnet->find(true);

                $vlanname = $subnet->vlanname;

                $vlan->free();
                $subnet->free();
            }

            // now take the host's information and write it to the config file
            if ($vlanname != '') {
                if ($mode['debug']) {
                    fwrite ($fh, "address {$dotted_ha} vlan-name {$vlanname} ! {$hosts->name}.{$hosts->domainname}\n");
                } else {
                    fwrite ($fh, "address {$dotted_ha} vlan-name {$vlanname}\n");
                }
            }
        }
        $domain->free();
        $hosts->free();
    }
// }}}

// {{{ _include_file
/**
 * include a template file into the output data file
 *
 * @param filehandler of output file
 * @param string filename of file to be included
 * @return boolean success
 * @access private
 */
    function _include_file(&$fh, $incfile)
    {
        // try to open include template and put it into the config file
        if (!$inc_fh = @fopen($incfile, 'r')) {
            return false;
        } else {
            while (!feof($inc_fh)) {
                fwrite($fh, fgets($inc_fh));
            }
            fclose($inc_fh);
            return true;
        }
        
    }
// }}}

}

?>
