<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2005 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id$

/**
  * Contains the {@link DBBackup} class
  *
  * @author $Author$
  * @filesource
  *
  * @package    classes
  * @subpackage backend
  */

require_once('PEAR.php');
require_once MODDIR.'/module.php';

/**
 * Provides a database backup/dump service to the backend
 *
 * @package classes
 * @subpackage backend
 */
class DBBackup extends Module
{
// {{{ vars
    var $log;           // object history DB_DataObject
    var $lastbackup;    // when was DB last backed up?
// }}}

/* Methods for Config File Intergrity Checking */
// {{{ needsConfigFile()

    /**
      * Returns, if the module needs a config file at all. This will be used
      * by the automated config file integrity checks, for example.
      * Defaults to false.
      * Can be called statically.
      *
      * @return boolean .cfg file vital for the module to work?
      */
    function needsConfigFile()
    {
        return true;
    }
// }}}

/* Module Overloads */
// {{{ _install()

    function _install()
    {
        return true;
    }

// }}}
// {{{ _uninstall()

    function _uninstall()
    {
        return true;
    }

// }}}
// {{{ initialize()

    function initialize()
    {
        // fetch last backup
        $this->log = DB_DataObject::Factory('object_history');
        $this->log->username = 'backend';
        $this->log->object_type = 'dbbackup';
        $this->log->action = 'dumpDB';
        $this->log->summary = 'success';
        $this->log->orderBy('date DESC');
        $this->log->limit(1);
        $numRows = $this->log->find(true);
        if ($numRows == 0) {
            $this->lastbackup = false;      // no prior backup found
        } else {
            // save prior timestamp for later use when backing up!
            $this->lastbackup = $this->log->date;
        }
    }

// }}}
// {{{ register_hooks()
    /**
      * Connect this module to the module hooks it should reply to
      */
    function register_hooks()
    {
        Module::add_action('backend_backup', 'dumpDB');
        Module::add_action('backend_maintenance', 'wipeDumps');
    }
// }}}

/* Private Module Functions */
// {{{ dumpDB()
/**
 * do actual database backup. Triggered by runBackend() on backup time.
 *
 * @access private
 */
    function dumpDB() {
        // make a new name for a database dump
        $backuptime = time();
        $newdump = BUILD . '/backups/sql/' . date('Y-m-d-His', $backuptime);
        
        system(exec('which nice').' '.exec('which mysqldump')." -u{$this->config['dbbackup']['database']['username']} -p{$this->config['dbbackup']['database']['password']} --allow-keywords -h{$this->config['dbbackup']['database']['host']} -Q --add-drop-table {$this->config['dbbackup']['database']['database']} > ${newdump}.sql");

        // compress the dump
        switch ($this->config['dbbackup']['backups_compress']) {
        case 'bzip2':
        case 'gzip':
            system(exec('which nice').' '.exec("which {$this->config['dbbackup']['backups_compress']}")." ${newdump}.sql");
            break;
            
        default:
            break;
        }

        // now log successful backup
        $this->log->free();     // make sure it's empty
        $this->log->username = 'backend';
        $this->log->object_type = 'dbbackup';
        $this->log->date = $backuptime;
        $this->log->comment = "Successfully dumped database to ${newdump}.sql";
        $this->log->action = 'dumpDB';
        $this->log->summary = 'success';
        $this->log->insert();
    }
// }}}
// {{{ wipeDumps()
/**
 * wipe expired database dumps. Triggered by runBackend() at maintenance time.
 *
 * @access private
 */
    function wipeDumps()
    {
        $wipetime = time();
        $dumpdirname = BUILD . '/backups/sql/';
        
        $dumpswiped = 0;

        // go through dumps and wipe them if there are any expired ones
        $dumpdir = opendir($dumpdirname);
        if (!$dumpdir) PEAR::raiseError('DBBackup module could not open dump dir: '.$dumpdirname);
        while (false !== ($dumpfile = readdir ($dumpdir))) {
            if (strstr($dumpfile, '.sql')) { // only dump files
                if ($wipetime - filemtime($dumpdirname.$dumpfile) > 60*60*24*$this->config['dbbackup']['backups_keep']) {
                    // expired? kick it.
                    if (!unlink($dumpdirname.$dumpfile)) {
                        PEAR::raiseError("DBBackup module could not wipe expired dump $dumpfile");
                    } else {
                        $dumpswiped++;
                    }
                }
            }
        }
        closedir($dumpdir);
        
        // now log successful wiping if any
        if ($dumpswiped > 0) {
            $this->log->free();     // make sure it's empty
            $this->log->username = 'backend';
            $this->log->object_type = 'dbbackup';
            $this->log->date = $wipetime;
            $this->log->comment = "Successfully wiped ${dumpswiped} expired database dumps";
            $this->log->action = 'wipeDumps';
            $this->log->summary = 'success';
            $this->log->insert();
        }
    }
// }}}

}

?>
